<?php

define('EXPAPAY_MODXMOD_VER', 1.0);
define('EXPAPAY_PURCHASE_URL', 'https://gw.expapay.com/v1/purchase');

if (!class_exists('msPaymentInterface')) {
	require_once MODX_BASE_PATH."/core/components/minishop2/model/minishop2/mspaymenthandler.class.php";
}

function flatten($array, $prefix="") {
	$result = array();
	foreach($array as $key => $value) {
		if(is_array($value)) $result = $result + flatten($value, "{$prefix}{$key}.");
		else $result[$prefix . $key] = $value;
		}
	return $result;
}

class Expapay extends msPaymentHandler implements msPaymentInterface {
	function __construct(xPDOObject $object, $config = array()) {
		$this->modx = &$object->xpdo;
		$this->config = array_merge(array(
			'result_url' => $resultUrl,
			'account_id' => $this->modx->getOption(
				'ms2expapay_account_id', null,
				'5c20b96445e5f318a4a9b2e4', true
			),
			'signing_key' => $this->modx->getOption(
				'ms2expapay_signing_key', null,
				'mVOBjC1zTS2m-7l9MLZ3Hw', true
			), 
		), $config);
	}

	private function __get_signature($request_data) {
		$temp = flatten($request_data);
		ksort($temp);
		return hash_hmac('sha256', join('', $temp), $this->config['signing_key']);
	}
	
	public function send(msOrder $order) {
		$amount = $order->get('cost');
		$payer_profile = $order->getOne('User')->getOne('Profile');

		$request_data = array(
			'account_id' => $this->config['account_id'],
			'order_id' => (string)$order->get('id'),
			'amount' => $amount,
			'payer' => array('email' => $payer_profile->email)
		);

		$request_data['signature'] = $this->__get_signature($request_data);

		$options = array(
			'http' => array(
				'header'  => "Content-type: application/json\r\n",
				'user_agent' => 'Modx-MiniShop2-plugin-v' . EXPAPAY_MODXMOD_VER,
				'method'  => 'POST',
				'content' => json_encode($request_data),
				'ignore_errors' => true
			),
		);

		$context = stream_context_create($options);
		$response = file_get_contents(EXPAPAY_PURCHASE_URL, false, $context);
		$response = json_decode($response);

		if (array_key_exists('error', $response)) var_dump($response);
		else return $this->success('', array('redirect' => $response->redirect_to));
	}

	public function receive(msOrder $order, $params = array()) {
		$order_id = $order->get('id');

		$signature = $params['signature'];
		unset($params['signature']);

		if ($this->__get_signature($params) != $signature)
			return $this->error("Invalid signature '$signature' reveived for order id $order_id");

		@$this->modx->context->key = 'mgr';
		$miniShop2 = $this->modx->getService('miniShop2');

		if ('CAPTURED' == $params['status']) {
			$miniShop2->changeOrderStatus($order_id, 2);
			$page_id = $this->modx->getOption('ms2expapay_success_id', null, 1, true);
		}

		if (in_array($params['status'], array('FAILED', 'REJECTED', 'CANCELLED'))) {
			$miniShop2->changeOrderStatus($order_id, 4);
			$page_id = $this->modx->getOption('ms2expapay_cancel_id', null, 1, true);
		}

		$context = $order->get('context');
		$params = array('msorder' => $order_id);
		return $this->modx->makeUrl($page_id, $context, $params, 'full');
	}
}
